/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#define EB 0x00
#define EL 0x01

static int host_endianness;
static int rawx_endianness;

static int sc_open(FILE**, const char*, struct stat*);
static void usage(char*);

void endianness_detect(void)
{
   unsigned int ie = 0x00000001;

   if(*(unsigned char*)&ie){
      printf("[little endian host]\n");
      host_endianness = EL;
   }else{
      printf("[big endian host]\n");
      host_endianness = EB;
   }
}

unsigned int swap32(unsigned int xe)
{
	int i=4;
	unsigned int ye = 0;
   
	while(i--){
		ye <<= 8;
		ye |= xe & 0xff;
		xe >>= 8;
	}

	return ye;
}

struct rawx_hdr{
	unsigned char magic[4];
	unsigned int width;
	unsigned int height;
	unsigned int mode;	/* ARGB32 */
	unsigned int reserved;
} __attribute__((__packed__));

static struct bmp_file_hdr* bmp_file_info;
static struct dib_hdr* dib_info;

int rawx_id(unsigned char* rawx_header)
{
	unsigned char *c;
	struct rawx_hdr* rawx_info;

	rawx_info = (struct rawx_hdr*)rawx_header;

	if(!strncmp((char*)rawx_info->magic, "RAWL", 4)){
		printf("format  : little endian\n");
		rawx_endianness = EL;
	}else if(!strncmp((char*)rawx_info->magic, "RAWB", 4)){
		printf("format: big endian\n");
		rawx_endianness = EB;
	}else{
		printf("error: invalid or damaged rawx file\n");
		return -1;
	}

	if(rawx_endianness != host_endianness)
	{
		printf("geometry: %dx%dx%d\n", \
							   swap32(rawx_info->width),
							   swap32(rawx_info->height),
							   swap32(rawx_info->mode) /* ... */);
	}else{
		printf("geometry: %dx%dx%d\n", \
									   rawx_info->width,
									   rawx_info->height,
									   rawx_info->mode /* ... */);
	}

	/* debug */
	c = (unsigned char *)rawx_info;
	printf("__rawx header dump__\n");
	printf("magic   : %c%c%c%c\n", *(c), *(c+1), *(c+2), *(c+3)); c+=4;
	printf("width   : 0x%02x%02x%02x%02x\n", *(c), *(c+1), *(c+2), *(c+3)); c+=4;
	printf("height  : 0x%02x%02x%02x%02x\n", *(c), *(c+1), *(c+2), *(c+3)); c+=4;
	printf("mode    : 0x%02x%02x%02x%02x\n", *(c), *(c+1), *(c+2), *(c+3)); c+=4;
	printf("reserved: 0x%02x%02x%02x%02x\n", *(c), *(c+1), *(c+2), *(c+3)); c+=4;

}

int main(int argc, char* argv[])
{
	FILE* fd0;
	struct stat info;

	unsigned char* rawx_buf;
	unsigned long address;

	if(argc < 2){
		usage(argv[0]);
		return -1;
	}else{
		if( sc_open(&fd0, argv[1], &info) ) return -1;
	}

	/* min 20 bytes (file header) */

	if(info.st_size < 20){
		printf("error: file is too small [%d]\n", info.st_size);
		return -1;
	}

	rawx_buf = malloc(20);
	if(!rawx_buf){
		printf("error: [alloc. %d] failed\n", (int)info.st_size);
		return -1;
	}else{
		/* printf("[alloc. %d]\n", (int)20); */
	}

	/* ... */

	endianness_detect();
	fread(rawx_buf, 1, 20, fd0);
	rawx_id(rawx_buf);

	/* ... */

	free(rawx_buf);
	fclose(fd0);

	return 0;
}

static int sc_open(FILE** fd, const char* name, struct stat* info)
{
      if (stat(name, info) == -1)
      {
          if (errno == ENOENT)
          {
            (void) fprintf( stderr, "error: '%s' does not exist\r\n", name );
          }else{
            (void) fprintf( stderr, "error: stat() fails for '%s'\r\n", name );
          }
          return -1;
      }

      /* checking S_ISLNK(m) is not required since 'stat' follows sym. links */
      if (!S_ISREG(info->st_mode))
      {
         (void) fprintf( stderr, "error: '%s' is a not a regular file (or a sym. link)\r\n", name);
         return -1;
      }

      if ((*fd = fopen(name, "r")) == NULL)
      {
         (void) fprintf( stderr, "error: '%s' open failed\r\n", name );
         return -1;
      }

	  /* printf("size: %d\n", (int)info->st_size); */

      return 0;   /* no error */
}

static void usage(char* s)
{
	printf("usage: %s input.bmp\n", s);
}
